@@ -20,7 +20,7 @@ class Analyzer(sc.prettyobj):
     Base class for analyzers. Based on the Intervention class.
 
     Args:
-        label (str): a label for the intervention (used for ease of identification)
+        label (str): a label for the Analyzer (used for ease of identification)
     '''
 
     def __init__(self, label=None):
@@ -48,6 +48,23 @@ class Analyzer(sc.prettyobj):
         raise NotImplementedError
 
 
+def validate_recorded_dates(sim, requested_dates, recorded_dates, die=True):
+    '''
+    Helper method to ensure that dates recorded by an analyzer match the ones
+    requested.
+    '''
+    requested_dates = sorted(list(requested_dates))
+    recorded_dates = sorted(list(recorded_dates))
+    if recorded_dates != requested_dates:
+        errormsg = f'The dates {requested_dates} were requested but only {recorded_dates} were recorded: please check the dates fall between {sim.date(sim["start_day"])} and {sim.date(sim["start_day"])} and the sim was actually run'
+        if die:
+            raise RuntimeError(errormsg)
+        else:
+            print(errormsg)
+    return
+
+
+
 class snapshot(Analyzer):
     '''
     Analyzer that takes a "snapshot" of the sim.people array at specified points
@@ -55,8 +72,10 @@ class snapshot(Analyzer):
     the dictionary directly, or use the get() method.
 
     Args:
-        days (list): list of ints/strings/date objects, the days on which to take the snapshot
-        kwargs (dict): passed to Intervention()
+        days   (list): list of ints/strings/date objects, the days on which to take the snapshot
+        args   (list): additional day(s)
+        die    (bool): whether or not to raise an exception if a date is not found (default true)
+        kwargs (dict): passed to Analyzer()
 
 
     **Example**::
@@ -71,11 +90,12 @@ class snapshot(Analyzer):
         people = snapshot.get()                   # Option 5
     '''
 
-    def __init__(self, days, *args, **kwargs):
-        super().__init__(**kwargs) # Initialize the Intervention object
+    def __init__(self, days, *args, die=True, **kwargs):
+        super().__init__(**kwargs) # Initialize the Analyzer object
         days = sc.promotetolist(days) # Combine multiple days
         days.extend(args) # Include additional arguments, if present
         self.days      = days # Converted to integer representations
+        self.die       = die  # Whether or not to raise an exception
         self.dates     = None # String representations
         self.start_day = None # Store the start date of the simulation
         self.snapshots = sc.odict() # Store the actual snapshots
@@ -84,8 +104,12 @@ class snapshot(Analyzer):
 
     def initialize(self, sim):
         self.start_day = sim['start_day'] # Store the simulation start day
-        self.days = cvi.process_days(sim, self.days) # Ensure days are in the right format
-        self.dates = [sim.date(day) for day in self.days] # Store as date strings
+        self.days, self.dates = cvi.process_days(sim, self.days, return_dates=True) # Ensure days are in the right format
+        max_snapshot_day = self.days[-1]
+        max_sim_day = sim.day(sim['end_day'])
+        if max_snapshot_day > max_sim_day:
+            errormsg = f'Cannot create snapshot for {self.dates[-1]} (day {max_snapshot_day}) because the simulation ends on {self.end_day} (day {max_sim_day})'
+            raise ValueError(errormsg)
         self.initialized = True
         return
 
@@ -94,6 +118,11 @@ class snapshot(Analyzer):
         for ind in cvi.find_day(self.days, sim.t):
             date = self.dates[ind]
             self.snapshots[date] = sc.dcp(sim.people) # Take snapshot!
+
+        # On the final timestep, check that everything matches
+        if sim.t == sim.tvec[-1]:
+            validate_recorded_dates(sim, requested_dates=self.dates, recorded_dates=self.snapshots.keys(), die=self.die)
+
         return
 
 
@@ -112,6 +141,7 @@ class snapshot(Analyzer):
         return snapshot
 
 
+
 class age_histogram(Analyzer):
     '''
     Analyzer that takes a "snapshot" of the sim.people array at specified points
@@ -125,7 +155,8 @@ class age_histogram(Analyzer):
         edges   (list): edges of age bins to use (default: 10 year bins from 0 to 100)
         datafile (str): the name of the data file to load in for comparison, or a dataframe of data (optional)
         sim      (Sim): only used if the analyzer is being used after a sim has already been run
-        kwargs  (dict): passed to Intervention()
+        die     (bool): whether to raise an exception if dates are not found (default true)
+        kwargs  (dict): passed to Analyzer()
 
     **Examples**::
 
@@ -136,12 +167,13 @@ class age_histogram(Analyzer):
         agehist = cv.age_histogram(sim=sim)
     '''
 
-    def __init__(self, days=None, states=None, edges=None, datafile=None, sim=None, **kwargs):
-        super().__init__(**kwargs) # Initialize the Intervention object
+    def __init__(self, days=None, states=None, edges=None, datafile=None, sim=None, die=True, **kwargs):
+        super().__init__(**kwargs) # Initialize the Analyzer object
         self.days      = days # To be converted to integer representations
         self.edges     = edges # Edges of age bins
         self.states    = states # States to save
         self.datafile  = datafile # Data file to load
+        self.die       = die # Whether to raise an exception if dates are not found
         self.bins      = None # Age bins, calculated from edges
         self.dates     = None # String representations of dates
         self.start_day = None # Store the start date of the simulation
@@ -153,16 +185,24 @@ class age_histogram(Analyzer):
         return
 
 
+    def from_sim(self, sim):
+        ''' Create an age histogram from an already run sim '''
+        if self.days is not None:
+            errormsg = 'If a simulation is being analyzed post-run, no day can be supplied: only the last day of the simulation is available'
+            raise ValueError(errormsg)
+        self.initialize(sim)
+        self.apply(sim)
+        return
+
+
     def initialize(self, sim):
 
         # Handle days
         self.start_day = cvm.date(sim['start_day'], as_date=False) # Get the start day, as a string
-        self.end_day   = cvm.date(sim['end_day'], as_date=False) # Get the start day, as a string
+        self.end_day   = cvm.date(sim['end_day'],   as_date=False) # Get the start day, as a string
         if self.days is None:
             self.days = self.end_day # If no day is supplied, use the last day
-        self.days = cvi.process_days(sim, self.days) # Ensure days are in the right format
-        self.days = np.sort(self.days) # Ensure they're in order
-        self.dates = [sim.date(day) for day in self.days] # Store as date strings
+        self.days, self.dates = cvi.process_days(sim, self.days, return_dates=True) # Ensure days are in the right format
         max_hist_day = self.days[-1]
         max_sim_day = sim.day(self.end_day)
         if max_hist_day > max_sim_day:
@@ -204,6 +244,11 @@ class age_histogram(Analyzer):
             for state in self.states: # Loop over each state
                 inds = sim.people.defined(f'date_{state}') # Pull out people for which this state is defined
                 self.hists[date][state] = np.histogram(age[inds], bins=self.edges)[0]*scale # Actually count the people
+
+        # On the final timestep, check that everything matches
+        if sim.t == sim.tvec[-1]:
+            validate_recorded_dates(sim, requested_dates=self.dates, recorded_dates=self.hists.keys(), die=self.die)
+
         return
 
 
@@ -244,13 +289,6 @@ class age_histogram(Analyzer):
         return
 
 
-    def from_sim(self, sim):
-        ''' Create an age histogram from an already run sim '''
-        self.initialize(sim)
-        self.apply(sim)
-        return
-
-
     def plot(self, windows=False, width=0.8, color='#F8A493', font_size=18, fig_args=None, axis_args=None, data_args=None):
         '''
         Simple method for plotting the histograms.
@@ -319,16 +357,17 @@ class Fit(sc.prettyobj):
         - difference: the absolute numerical differences between the model and the data (one time series per result)
         - goodness-of-fit: the result of passing the difference through a statistical function, such as mean squared error
         - loss: the goodness-of-fit for each result multiplied by user-specified weights (one time series per result)
-        - mismatch: the sum of all the loses (a single scalar value) -- this is the value to be minimized during calibration
+        - mismatches: the sum of all the losses (a single scalar value per time series)
+        - mismatch: the sum of the mismatches -- this is the value to be minimized during calibration
 
     Args:
         sim (Sim): the sim object
-        weights (dict): the relative weight to place on each result
+        weights (dict): the relative weight to place on each result (by default: 10 for deaths, 5 for diagnoses, 1 for everything else)
         keys (list): the keys to use in the calculation
-        method (str): the method to be used to calculate the goodness-of-fit
-        custom (dict): a custom dictionary of additional data to fit; format is e.g. {'<label>':{'data':[1,2,3], 'sim':[1,2,4], 'weights':2.0}}
+        custom (dict): a custom dictionary of additional data to fit; format is e.g. {'my_output':{'data':[1,2,3], 'sim':[1,2,4], 'weights':2.0}}
         compute (bool): whether to compute the mismatch immediately
         verbose (bool): detail to print
+        kwargs (dict): passed to cv.compute_gof() -- see this function for more detail on goodness-of-fit calculation options
 
     **Example**::
 
@@ -338,14 +377,15 @@ class Fit(sc.prettyobj):
         fit.plot()
     '''
 
-    def __init__(self, sim, weights=None, keys=None, method=None, custom=None, compute=True, verbose=False):
+    def __init__(self, sim, weights=None, keys=None, custom=None, compute=True, verbose=False, **kwargs):
 
         # Handle inputs
-        self.weights = weights
-        self.custom  = sc.mergedicts(custom)
-        self.verbose = verbose
-        self.weights = sc.mergedicts({'cum_deaths':10, 'cum_diagnoses':5}, weights)
-        self.keys    = keys
+        self.weights    = weights
+        self.custom     = sc.mergedicts(custom)
+        self.verbose    = verbose
+        self.weights    = sc.mergedicts({'cum_deaths':10, 'cum_diagnoses':5}, weights)
+        self.keys       = keys
+        self.gof_kwargs = kwargs
 
         # Copy data
         if sim.data is None:
@@ -482,6 +522,7 @@ class Fit(sc.prettyobj):
 
     def compute_gofs(self, **kwargs):
         ''' Compute the goodness-of-fit '''
+        kwargs = sc.mergedicts(self.gof_kwargs, kwargs)
         for key in self.pair.keys():
             actual    = sc.dcp(self.pair[key].data)
             predicted = sc.dcp(self.pair[key].sim)
@@ -555,8 +596,8 @@ class Fit(sc.prettyobj):
         main_ax1 = pl.subplot(n_rows, 2, 1)
         main_ax2 = pl.subplot(n_rows, 2, 2)
         bottom = sc.objdict() # Keep track of the bottoms for plotting cumulative
-        bottom.a = np.zeros(self.losses[0].shape)
-        bottom.b = np.zeros(self.losses[0].shape)
+        bottom.daily = np.zeros(self.sim_npts)
+        bottom.cumul = np.zeros(self.sim_npts)
         for k,key in enumerate(keys):
             if key in self.keys: # It's a time series, plot with days and dates
                 days      = self.inds.sim[key] # The "days" axis (or not, for custom keys)
@@ -579,12 +620,12 @@ class Fit(sc.prettyobj):
                         title = f'Cumulative mismatch: {self.mismatch:0.3f}'
 
                     dates = self.sim_results['date'][days] # Show these with dates, rather than days, as a reference point
-                    ax.bar(dates, data, width=width, bottom=bottom[i], color=colors[k], label=f'{key}')
+                    ax.bar(dates, data, width=width, bottom=bottom[i][self.inds.sim[key]], color=colors[k], label=f'{key}')
 
                     if i == 0:
-                        bottom[i] += self.losses[key]
+                        bottom.daily[self.inds.sim[key]] += self.losses[key]
                     else:
-                        bottom[i] += np.cumsum(self.losses[key])
+                        bottom.cumul = np.cumsum(bottom.daily)
 
                     if k == len(self.keys)-1:
                         ax.set_xlabel('Date')
@@ -774,7 +815,7 @@ class TransTree(sc.prettyobj):
                 stdict = {'t':target}
 
             # Pull out each of the attributes relevant to transmission
-            attrs = ['age', 'date_symptomatic', 'date_tested', 'date_diagnosed', 'date_quarantined', 'date_severe', 'date_critical', 'date_known_contact']
+            attrs = ['age', 'date_exposed', 'date_symptomatic', 'date_tested', 'date_diagnosed', 'date_quarantined', 'date_severe', 'date_critical', 'date_known_contact']
             for st,stind in stdict.items():
                 for attr in attrs:
                     ddict[st][attr] = people[attr][stind]
@@ -993,7 +1034,7 @@ class TransTree(sc.prettyobj):
         for day in range(n):
             pl.title(f'Day: {day}')
             pl.xlim([0, n])
-            pl.ylim([0, len(self)])
+            pl.ylim([0, self.pop_size])
             pl.xlabel('Day')
             pl.ylabel('Person')
             flist = frames[day]
@@ -1048,8 +1089,6 @@ class TransTree(sc.prettyobj):
 
         bins = bins[:-1] # Remove last bin since it's an edge
         total_counts = counts*bins
-        # counts = counts*100/counts.sum()
-        # total_counts = total_counts*100/total_counts.sum()
         n_bins = len(bins)
         index = np.linspace(0, 100, len(n_targets))
         sorted_arr = np.sort(n_targets)
