@@ -226,7 +226,7 @@ class BaseSim(ParsObj):
         features.
 
         Args:
-            ind (int, list, or array): the day(s) in simulation time
+            ind (int, list, or array): the index day(s) in simulation time (NB: strings and date objects are accepted, and will be passed unchanged)
             args (list): additional day(s)
             dateformat (str): the format to return the date in
             as_date (bool): whether to return as a datetime date instead of a string
@@ -236,13 +236,15 @@ class BaseSim(ParsObj):
 
         **Examples**::
 
+            sim = cv.Sim()
             sim.date(34) # Returns '2020-04-04'
             sim.date([34, 54]) # Returns ['2020-04-04', '2020-04-24']
-            sim.date(34, 54, as_dt=True) # Returns [datetime.date(2020, 4, 4), datetime.date(2020, 4, 24)]
+            sim.date([34, '2020-04-24']) # Returns ['2020-04-04', '2020-04-24']
+            sim.date(34, 54, as_date=True) # Returns [datetime.date(2020, 4, 4), datetime.date(2020, 4, 24)]
         '''
 
         # Handle inputs
-        if sc.isnumber(ind): # If it's a number, convert it to a list
+        if not isinstance(ind, list): # If it's a number, string, or dateobj, convert it to a list
             ind = sc.promotetolist(ind)
         ind.extend(args)
         if dateformat is None:
@@ -250,8 +252,11 @@ class BaseSim(ParsObj):
 
         # Do the conversion
         dates = []
-        for i in ind:
-            date_obj = cvm.date(self['start_day']) + dt.timedelta(days=int(i))
+        for raw in ind:
+            if sc.isnumber(raw):
+                date_obj = cvm.date(self['start_day'], as_date=True) + dt.timedelta(days=int(raw))
+            else:
+                date_obj = cvm.date(raw, as_date=True)
             if as_date:
                 dates.append(date_obj)
             else:
@@ -447,9 +452,9 @@ class BaseSim(ParsObj):
             shrunken_sim (Sim): a Sim object with the listed attributes removed
         '''
 
-        # By default, skip people (~90% of memory) and the popdict (which is usually empty anyway)
+        # By default, skip people (~90% of memory), the popdict (which is usually empty anyway), and _orig_pars (which is just a backup)
         if skip_attrs is None:
-            skip_attrs = ['popdict', 'people']
+            skip_attrs = ['popdict', 'people', '_orig_pars']
 
         # Create the new object, and copy original dict, skipping the skipped attributes
         if in_place:
@@ -584,7 +589,13 @@ class BasePeople(sc.prettyobj):
 
 
     def __getitem__(self, key):
-        ''' Allow people['attr'] instead of getattr(people, 'attr') '''
+        ''' Allow people['attr'] instead of getattr(people, 'attr')
+            If the key is an integer, alias `people.person()` to return a `Person` instance
+        '''
+
+        if isinstance(key, int):
+            return self.person(key)
+
         try:
             return self.__dict__[key]
         except:
@@ -607,8 +618,9 @@ class BasePeople(sc.prettyobj):
 
 
     def __iter__(self):
-        ''' Define the iterator to just be the indices of the array '''
-        return iter(range(len(self)))
+        ''' Iterate over people '''
+        for i in range(len(self)):
+            yield self[i]
 
 
     def __add__(self, people2):
@@ -794,11 +806,7 @@ class BasePeople(sc.prettyobj):
 
     def to_people(self):
         ''' Return all people as a list '''
-        people = []
-        for p in self:
-            person = self.person(p)
-            people.append(person)
-        return people
+        return list(self)
 
 
     def from_people(self, people, resize=True):
@@ -936,8 +944,8 @@ class Person(sc.prettyobj):
         self.age         = cvd.default_float(age) # Age of the person (in years)
         self.sex         = cvd.default_int(sex) # Female (0) or male (1)
         self.contacts    = contacts # Contacts
-        self.infected = [] #: Record the UIDs of all people this person infected
-        self.infected_by = None #: Store the UID of the person who caused the infection. If None but person is infected, then it was an externally seeded infection
+        # self.infected = [] #: Record the UIDs of all people this person infected
+        # self.infected_by = None #: Store the UID of the person who caused the infection. If None but person is infected, then it was an externally seeded infection
         return
 
 
@@ -1095,3 +1103,34 @@ class Layer(FlexDict):
             self[key] = df[key].to_numpy()
         return self
 
+
+    def find_contacts(self, inds, as_array=True):
+        """
+        Find all contacts of the specified people
+
+        For some purposes (e.g. contact tracing) it's necessary to find all of the contacts
+        associated with a subset of the people in this layer. Since contacts are bidirectional
+        it's necessary to check both P1 and P2 for the target indices. The return type is a Set
+        so that there is no duplication of indices (otherwise if the Layer has explicit
+        symmetric interactions, they could appear multiple times). This is also for performance so
+        that the calling code doesn't need to perform its own unique() operation. Note that
+        this cannot be used for cases where multiple connections count differently than a single
+        infection, e.g. exposure risk.
+
+        Args:
+            inds (array): indices of people whose contacts to return
+            as_array (bool): if true, return as sorted array (otherwise, return as unsorted set)
+
+        Returns:
+            contact_inds (array): a set of indices for pairing partners
+
+        Example: If there were a layer with
+        - P1 = [1,2,3,4]
+        - P2 = [2,3,1,4]
+        Then find_contacts([1,3]) would return {1,2,3}
+        """
+        contact_inds = cvu.find_contacts(self['p1'], self['p2'], inds)
+        if as_array:
+            contact_inds = np.fromiter(contact_inds, dtype=cvd.default_int)
+            contact_inds.sort()  # Sorting ensures that the results are reproducible for a given seed as well as being identical to previous versions of Covasim
+        return contact_inds
